﻿using Capnp.Rpc;
using FabAccessAPI.Schema;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace FabAccessAPI {
    public class Connection {
        #region private variables
        private TcpRpcClient? _rpcClient = null;
        private IBootstrap? _bootstrapCap = null;
        private AuthUser? _authUser = null;
        private Auth? _auth = null;
        private Machines? _machines = null;
        #endregion
        
        #region Log
        private static readonly log4net.ILog _Log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        #endregion
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="rpcClient">Should be an already configured and connected TcpRpcClient</param>
        public Connection(TcpRpcClient rpcClient) {
            _rpcClient = rpcClient;
            _bootstrapCap = _rpcClient.GetMain<IBootstrap>();
            _Log.Debug($"Done bootstraping API connection.");
        }

        /// <summary>
        /// Authenticate this connection.
        /// Calling this more then once is UB
        /// </summary>
        /// <param name="mech">The desired authentication mechanism</param>
        /// <param name="kvs">Key-Value data specific to the mechanism</param>
        /// <returns></returns>
        public async Task Auth(string mech, Dictionary<string, object> kvs) {
            // _bootstrapCap = await _bootstrapCap.Unwrap();
            var authCap = await _bootstrapCap.Auth();
            _auth = new Auth(authCap);
            var mechs = await _auth.GetMechanisms().ConfigureAwait(false);
            _Log.Debug($"The Server supports the following auth mechs: {mechs}");

            // TODO: Check that the requested auth mech is actually available.

            await _auth.Authenticate(mech, kvs).ConfigureAwait(false);
        }

        /// <summary>
        /// Get a wrapped capability to interact with machines
        /// </summary>
        /// <returns>A wrapped capability to interact with machines</returns>
        public async Task<Machines> AccessMachines() {
            _machines ??= new Machines((await _bootstrapCap.Machines().ConfigureAwait(false)));
            return _machines;
        }
    }
}
